#include "stdafx.h"
#include "avifile.h"
#include "images.h"
#include "cximage/cximage/ximage.h"


// Default values
int FrameRate=30;	// Encoding a file with this framerate
BOOLEAN MultiThread=FALSE;
//int AviRef=0;


void Init()
{
//	if (AviRef==0)
	{
//		CoInitializeEx(NULL,COINIT_MULTITHREADED);

		// Initialise windows AVI engine
		AVIFileInit();
	}
//	AviRef++;
}

void Done()
{
//	AviRef--;
//	if (AviRef==0)
	{
		// Uninitialise windows AVI engine
		AVIFileExit();

//		CoUninitialize();
	}
}



CAviWrite::CAviWrite(HWND hWnd)
{
	// Set defaults
	aopts[0]=&opts;
	pfile=NULL;
	ps=NULL;
	psCompressed=NULL;
	index=0;
	owner=hWnd;

	// initialise avi engine
	Init();
}

CAviWrite::~CAviWrite()
{
	// Clean up, safely
	if (ps)
		AVIStreamClose(ps);

	if (psCompressed)
		AVIStreamClose(psCompressed);

	if (pfile)
		AVIFileClose(pfile);

	// uninitialise avi engine
	Done();
}

BOOLEAN CAviWrite::Create(PCHAR file)
{
	// Creates a new AVI file
	HRESULT hr;
	hr=AVIFileOpen(&pfile,
		       file,
		       OF_WRITE | OF_CREATE |OF_SHARE_EXCLUSIVE,
		       NULL);

#ifdef _DEBUG
	const HRESULT e1=AVIERR_BADFORMAT;
	const HRESULT e2=AVIERR_MEMORY;
	const HRESULT e3=AVIERR_FILEREAD;
	const HRESULT e4=AVIERR_FILEOPEN;
	int e=GetLastError();
#endif

	if (hr!=AVIERR_OK)return FALSE;

	return TRUE;
}

BOOLEAN CAviWrite::InitVideoStream(LPBITMAPINFOHEADER bih)
{
	// Creates a video stream for frames, depending on the format of the first loaded picture
	HRESULT hr;

	// Set framerate and destination size (of the first picture to encode)
	ZeroMemory(&strhdr,sizeof(strhdr));
	strhdr.fccType                = streamtypeVIDEO;
	strhdr.fccHandler             = 0;
	strhdr.dwScale                = 1;
	strhdr.dwRate                 = FrameRate;		    
	strhdr.dwSuggestedBufferSize  = bih->biSizeImage;
//	strhdr.dwFlags=AVISTREAMINFO_FORMATCHANGES;
	SetRect(&strhdr.rcFrame, 0, 0,		  
	    (int) bih->biWidth,
	    (int) bih->biHeight);

	width=bih->biWidth;
	height=bih->biHeight;
	bpp=bih->biBitCount;
	LPBITMAPINFO bi=(LPBITMAPINFO)bih;
	if (bpp<=8)
		memcpy(palette,bi->bmiColors,sizeof(RGBQUAD)*bih->biClrUsed);


	// And now create the stream;
	if (AVIFileCreateStream(pfile,		    // file pointer
			         &ps,		    // returned stream pointer
			         &strhdr)!=AVIERR_OK)
	{
		return FALSE;
	}

	// Now show compression dialog, where the user can select the codec
	ZeroMemory(&opts,sizeof(opts));

	if (!AVISaveOptions(owner, 0, 1, &ps, (LPAVICOMPRESSOPTIONS FAR *) &aopts))
	{
		return FALSE;
	}

	psCompressed=NULL;
	const DWORD e1=AVIERR_NOCOMPRESSOR;
	int e=GetLastError();

	if ((hr=AVIMakeCompressedStream(&psCompressed, ps, &opts, NULL))!=AVIERR_OK)
	{
		return FALSE;
	}

	if (AVIStreamSetFormat(psCompressed, 0,
			       bih,	    // stream format
			       bih->biSize +   // format size
			       bih->biClrUsed * sizeof(RGBQUAD))!=AVIERR_OK)
	{
		return FALSE;
	}

	// Clean up

	return TRUE;
}

BOOLEAN CAviWrite::AddBitmap(LPBYTE bits,LPBITMAPINFOHEADER bi)
{
	if (ps==NULL)
	{	// Create video stream, if not already done
		if (!InitVideoStream(bi))return FALSE;
	}

	HRESULT hr;

	// let's write, like hell
/*	if (bi->biBitCount<=8 && index>0)
		if (AVIStreamSetFormat(psCompressed, index,
			       bi,	    // stream format
			       bi->biSize +   // format size
			       bi->biClrUsed * sizeof(RGBQUAD))!=AVIERR_OK)
	{
		return FALSE;
	}*/


	hr = AVIStreamWrite(psCompressed,	// stream pointer
			index,						// time of this frame
			1,							// number to write
			bits,
			bi->biSizeImage,						// size of this frame
			AVIIF_KEYFRAME,				// flags....
			NULL,
			NULL);

	// Clean up and increment the frame counter by 1
	index++;
	if (hr != AVIERR_OK)return FALSE;

	return TRUE;
}

BOOLEAN CAviWrite::AddFile(PCHAR file)
{
	CxImage *Image=LoadPicture(file);
	if (Image==NULL)return AddAudioFile(file);

	if (bpp<=8 && ps)
	{	// If paletted, assign a standa<rd palette
		Image->IncreaseBpp(24);
//		Image->SetStdPalette();
		Image->DecreaseBpp(bpp,true,palette);
	}else if (Image->GetBpp()!=bpp)Image->IncreaseBpp(bpp);

	if (ps) if((Image->GetHeight()!=(unsigned long)height)||(Image->GetWidth()!=(unsigned long)width))
	{
		Image->Resample(width,height);
	}


	HBITMAP dib;
	BITMAPINFO *bi;
	DIBSECTION bitmap;
	dib=Image->MakeBitmap(0);

	GetObject(dib,sizeof(bitmap),&bitmap);

	bi=(BITMAPINFO*)malloc(sizeof(BITMAPINFOHEADER)+Image->GetPaletteSize());

	memcpy(&bi->bmiHeader,&bitmap.dsBmih,sizeof(BITMAPINFOHEADER));
	memcpy(&bi->bmiColors,Image->GetPalette(),Image->GetPaletteSize());

	
	if (ps==NULL)
	{	// Create video stream, if not already done
		if (!InitVideoStream((LPBITMAPINFOHEADER)bi))return FALSE;
	}

	// Add picture to video stream

	AddBitmap((LPBYTE)bitmap.dsBm.bmBits,(LPBITMAPINFOHEADER)bi);

	free(bi);

	// Clean up and increment the frame counter by 1
	delete Image;
	DeleteObject(dib);

	return TRUE;
}




BOOLEAN CAviWrite::AddAudioData(LPVOID format,LPVOID data,long formatsize,long datasize)
{
	PAVISTREAM audio=NULL;
	AVISTREAMINFO strhdr;

	ZeroMemory(&strhdr,sizeof(strhdr));
	strhdr.fccType                = streamtypeAUDIO;
	strhdr.fccHandler             = 0;
	strhdr.dwScale=strhdr.dwSampleSize= ((WAVEFORMAT*)(format))->nBlockAlign;
	strhdr.dwRate                 = ((WAVEFORMAT*)(format))->nSamplesPerSec*strhdr.dwScale;		    
	strhdr.dwSuggestedBufferSize  = 0;
	strhdr.dwSampleSize			  = ((WAVEFORMAT*)(format))->nBlockAlign;
	
	if (AVIFileCreateStream(pfile,&audio,		   
			         &strhdr)!=AVIERR_OK)goto error;

	AVIStreamSetFormat(audio,0,format,formatsize);

	long samples,bytes;
	AVIStreamWrite(audio,0,datasize/strhdr.dwSampleSize,data,datasize,0,&samples,&bytes);

	// Ready, now clean up

	AVIStreamRelease(audio);
	return TRUE;
error:
	return FALSE;
}




BOOLEAN CAviWrite::AddAudioFile(PCHAR filename)
{
	// Loads and saves an wave form audio file into an audio stream
	HANDLE file=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,0,0);
	if (file==INVALID_HANDLE_VALUE)return FALSE;
	if (file==0)return FALSE;

	long l;
	CHAR b[5];
	b[4]='\0';
	DWORD read;
	LPVOID format=NULL,data=NULL;
	long formatsize,datasize;
	PAVISTREAM audio=NULL;

	// Check the file, if it is an acceptable RIFF WAVE file and extract format and data
	ReadFile(file,&b[0],4,&read,NULL);
	if (strcmp(b,"RIFF")!=0)goto error;
	
	ReadFile(file,&l,sizeof(l),&read,NULL);

	ReadFile(file,&b[0],4,&read,NULL);
	if (strcmp(b,"WAVE")!=0)goto error;

	ReadFile(file,&b[0],4,&read,NULL);

	// read format 
	ReadFile(file,&formatsize,sizeof(formatsize),&read,NULL);
	format=malloc(formatsize);

	ReadFile(file,format,formatsize,&read,NULL);

	// read data 
	ReadFile(file,&b[0],4,&read,NULL);
	if (strcmp(b,"data")!=0)goto error;

	ReadFile(file,&datasize,sizeof(datasize),&read,NULL);
	data=malloc(datasize);

	ReadFile(file,data,datasize,&read,NULL);
	if (DWORD(datasize)!=read)return FALSE;

	// All read, now create audio stream and write 

	AddAudioData(format,data,formatsize,datasize);

	
	free(format);
	free(data);
	CloseHandle(file);
	return TRUE;
error:
	if (format!=NULL)free(format);
	if (data!=NULL)free(data);
	CloseHandle(file);
	return FALSE;
}

